package gov.va.vss.model;

import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.NONE;
import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.PUBLIC_ONLY;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.builder.CompareToBuilder;
import org.apache.commons.lang3.builder.EqualsBuilder;
import org.apache.commons.lang3.builder.HashCodeBuilder;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;
import org.hibernate.annotations.Immutable;

import com.fasterxml.jackson.annotation.JsonAutoDetect;

import gov.va.shared.model.core.AbstractAuditedVersionedPersistent;

@Entity
@Immutable
@Table(name = "VolunteerAwards", schema = "vss")
@Cache(usage = CacheConcurrencyStrategy.READ_ONLY)
@JsonAutoDetect(fieldVisibility = NONE, getterVisibility = PUBLIC_ONLY, isGetterVisibility = PUBLIC_ONLY)
public class Award extends AbstractAuditedVersionedPersistent<Award> implements Comparable<Award> {
	private static final long serialVersionUID = 6904844123870655771L;

	// -------------------------------------- Fields

	private String name;
	private String code;
	private int hoursRequired;
	private String type;
	private int awardHours;

	// -------------------------------------- Business Methods

	@Transient
	public String getDisplayName() {
		return name;
	}
	
	@Transient
	public String getDisplayNameAbbreviated() {
		return StringUtils.abbreviate(getDisplayName(), 35);
	}
	
	// -------------------------------------- Common Methods

	@Override
	protected boolean requiredEquals(Award oo) {
		/*
		 * doubt all three of these are necessary but don't have data to confirm
		 * yet - CPB
		 */
		return new EqualsBuilder().append(code, oo.getCode()).append(name, oo.getName()).append(type, oo.getType())
				.isEquals();
	}

	@Override
	protected int requiredHashCode() {
		return new HashCodeBuilder().append(code).append(name).append(type).toHashCode();
	}

	@Override
	public int compareTo(Award o) {
		if (equals(o))
			return 0;

		return new CompareToBuilder().append(hoursRequired, o.getHoursRequired()).append(name, o.getName())
				.toComparison() > 0 ? 1 : -1;
	}
	
	@Override
	public String toString() {
		return getDisplayName();
	}

	// -------------------------------------- Accessor Methods

	@Column(name = "AwardName", length = 45, nullable = false)
	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	@Column(name = "AwardCode", length = 2, nullable = false)
	public String getCode() {
		return code;
	}

	public void setCode(String code) {
		this.code = code;
	}

	@Column(name = "RequiredHours", nullable = false)
	public int getHoursRequired() {
		return hoursRequired;
	}

	public void setHoursRequired(int hoursRequired) {
		this.hoursRequired = hoursRequired;
	}

	@Column(name = "AwardType", length = 1)
	public String getType() {
		return type;
	}

	public void setType(String type) {
		this.type = type;
	}

	@Column(nullable = false)
	public int getAwardHours() {
		return awardHours;
	}

	public void setAwardHours(int awardHours) {
		this.awardHours = awardHours;
	}

}
