package gov.va.vss.model;

import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.NONE;
import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.PUBLIC_ONLY;

import javax.persistence.AttributeOverride;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;

import org.apache.commons.lang3.builder.CompareToBuilder;
import org.apache.commons.lang3.builder.EqualsBuilder;
import org.apache.commons.lang3.builder.HashCodeBuilder;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;
import org.hibernate.annotations.Immutable;

import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.annotation.JsonIgnore;

import gov.va.shared.model.core.AbstractAuditedVersionedPersistent;
import gov.va.vss.model.facility.Facility;

@Entity
@Immutable
@Table(name = "StaffTitles", schema = "vss")
@Cache(usage = CacheConcurrencyStrategy.READ_ONLY)
@JsonAutoDetect(fieldVisibility = NONE, getterVisibility = PUBLIC_ONLY, isGetterVisibility = PUBLIC_ONLY)
@AttributeOverride(name = "name", column = @Column(name = "title"))
public class StaffTitle extends AbstractAuditedVersionedPersistent<StaffTitle> implements Comparable<StaffTitle> {

	private static final long serialVersionUID = 9103380479149326407L;

	private String title;
	private boolean volChiefSupervisor;
	private String description;
	private String scope;
	private Facility facility;
	private boolean chief;

	// ---------------------------------------- Common Methods

	@Override
	protected boolean requiredEquals(StaffTitle oo) {
		return new EqualsBuilder().append(getTitle(), oo.getTitle()).append(getScope(), oo.getScope())
				.append(nullSafeGetId(getFacility()), nullSafeGetId(oo.getFacility())).isEquals();
	}

	@Override
	protected int requiredHashCode() {
		return new HashCodeBuilder().append(getTitle()).append(getScope()).append(nullSafeGetId(getFacility()))
				.toHashCode();
	}

	@Override
	public int compareTo(StaffTitle o) {
		if (equals(o))
			return 0;
		return new CompareToBuilder().append(getScope(), o.getScope()).append(getTitle(), o.getTitle())
				.toComparison() > 0 ? 1 : -1;
	}

	// -------------------------------------- Accessor Methods

	@Column(length = 50, nullable = false)
	public String getTitle() {
		return title;
	}

	public void setTitle(String title) {
		this.title = title;
	}

	@Column(name = "IsVolChiefSupervisor")
	public boolean isVolChiefSupervisor() {
		return volChiefSupervisor;
	}

	public void setVolChiefSupervisor(boolean volChiefSupervisor) {
		this.volChiefSupervisor = volChiefSupervisor;
	}

	@Column(length = 250)
	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	@Column(length = 1)
	public String getScope() {
		return scope;
	}

	public void setScope(String scope) {
		this.scope = scope;
	}

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "FacilityFK", unique = true)
	@JsonIgnore
	public Facility getFacility() {
		return facility;
	}

	public void setFacility(Facility facility) {
		this.facility = facility;
	}

	@Column(name = "IsChief", nullable = false)
	public boolean isChief() {
		return chief;
	}

	public void setChief(boolean chief) {
		this.chief = chief;
	}

}
