package gov.va.vss.persistence.impl.requirement;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.persistence.Query;

import org.apache.commons.lang3.ArrayUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Repository;

import gov.va.vss.model.requirement.AbstractRequirement;
import gov.va.vss.model.requirement.FacilityRequirement;
import gov.va.vss.model.requirement.GlobalRequirement;
import gov.va.vss.persistence.dao.requirement.RequirementDAO;
import gov.va.vss.persistence.impl.GenericHibernateSortedDAOImpl;
import gov.va.vss.persistence.queryCustomization.QueryCustomization;

@Repository
public class RequirementDAOImpl extends GenericHibernateSortedDAOImpl<AbstractRequirement> implements RequirementDAO {
	@SuppressWarnings("unused")
	private static final Logger log = LoggerFactory.getLogger(RequirementDAOImpl.class);

	@Override
	public List<AbstractRequirement> findByCriteria(Long facilityId, String name, QueryCustomization... customization) {

		List<String> whereClauseItems = new ArrayList<>();
		Map<String, Object> params = new HashMap<>();

		StringBuilder sb = new StringBuilder("select r from "); //
		
		if (facilityId != null) { 
			sb.append(FacilityRequirement.class.getName()) //
				.append(" r left join fetch r.facility f "); //
			whereClauseItems.add("f.id = :facilityId ");
			params.put("facilityId", facilityId);
		} else {
			sb.append(GlobalRequirement.class.getName()) //
			.append(" r ");
		}
		
		QueryCustomization cust = ArrayUtils.isEmpty(customization) ? new QueryCustomization() : customization[0];
		cust.appendRemainingJoins(sb, "r");
		
		if (name != null) {
			whereClauseItems.add("lower(r.name) = :name ");
			params.put("name", name.toLowerCase());
		} else {			
			if (cust.getOrderBy() == null)
				cust.setOrderBy("r.name");
		}
		
		Query q = constructQuery(em, sb, whereClauseItems, params, null, cust);
		
		@SuppressWarnings("unchecked")
		List<AbstractRequirement> results = q.getResultList();
		return results;
		
	}
	

}
