package gov.va.vss.persistence.impl.requirement;

//import static gov.va.vss.model.lookup.RequirementStatus.RequirementStatusValue.UNMET;

import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.persistence.Query;

import org.apache.commons.lang3.ArrayUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import gov.va.shared.model.core.AbstractAuditedPersistent;
import gov.va.vss.model.facility.Facility;
import gov.va.vss.model.requirement.AbstractRequirement;
import gov.va.vss.model.requirement.GlobalRequirement;
import gov.va.vss.model.requirement.VolunteerRequirement;
import gov.va.vss.persistence.dao.requirement.RequirementDAO;
import gov.va.vss.persistence.dao.requirement.VolunteerRequirementDAO;
import gov.va.vss.persistence.impl.GenericHibernateDAOImpl;
import gov.va.vss.persistence.queryCustomization.QueryCustomization;

@Repository
public class VolunteerRequirementDAOImpl extends GenericHibernateDAOImpl<VolunteerRequirement>
		implements VolunteerRequirementDAO {
	@SuppressWarnings("unused")
	private static final Logger log = LoggerFactory.getLogger(VolunteerRequirementDAOImpl.class);

	@Autowired
	private RequirementDAO requirementDAO;

	@SuppressWarnings("unchecked")
	@Override
	public List<VolunteerRequirement> findByCriteria(long volunteerId, long facilityId,
			QueryCustomization... customization) {

		List<String> whereClauseItems = new ArrayList<>();
		Map<String, Object> params = new HashMap<>();

		StringBuilder sb = new StringBuilder("select vr from ").append(VolunteerRequirement.class.getName())
				.append(" vr join vr.volunteer v ").append("join fetch vr.requirement r ")
				.append("left join fetch r.facility f ");

		whereClauseItems.add("(TYPE(r) = :globalReqClass or f.id = :facilityId)");
		params.put("globalReqClass", GlobalRequirement.class);
		params.put("facilityId", facilityId);

		whereClauseItems.add("v.id = :volunteerId ");
		params.put("volunteerId", volunteerId);

		whereClauseItems.add("r.inactive = false");

		QueryCustomization cust = ArrayUtils.isEmpty(customization) ? new QueryCustomization() : customization[0];
		cust.appendRemainingJoins(sb, "vr");

		if (cust.getOrderBy() == null)
			cust.setOrderBy("TYPE(r) desc, r.name");

		Query q = constructQuery(em, sb, whereClauseItems, params, null, cust);
		if (cust.getRowLimitation() != null)
			q.setMaxResults(cust.getRowLimitation());

		return q.getResultList();
	}

	@Override
	public int bulkInsertForVolunteerIfNecessary(long volunteerId, long facilityIdForNewRequirements) {
		return em.createNativeQuery("INSERT INTO vss.VolunteerRequirement " //
				+ " (VolunteerFK, RequirementFK, RequirementDate, Comments, Ver, CREATED_BY, CREATED_DATE, MODIFIED_BY," //
				+ " 	MODIFIED_DATE, VSS_STD_RequirementStatusFK)" //
				+ " select :newVolunteerId, r.id, null ,null, 0,:myUser,:nowUTC, :myUser, :nowUTC, null" //:statusFK" 
				+ " from vss.Requirement r" //
				+ " where (r.type = 'G' or (r.type = 'F' and r.FacilityFK = :myFacilityId))" //
				+ " 	and not exists (" //
				+ "			select * from vss.VolunteerRequirement vr2" //
				+ "			where vr2.VolunteerFK = :newVolunteerId" //
				+ "			and vr2.RequirementFK = r.id)").setParameter("newVolunteerId", volunteerId)
				.setParameter("myUser", AbstractAuditedPersistent.getCurrentUserIdForAudit()) //
				.setParameter("nowUTC", ZonedDateTime.now(ZoneId.of("Z"))) //
				.setParameter("myFacilityId", facilityIdForNewRequirements) //
				/*.setParameter("statusFK", UNMET.getId())*/.executeUpdate();
	}

	// when volunteer adds assignment...

	@Override
	public int bulkInsertForNewRequirement(long requirementId) {

		AbstractRequirement requirement = requirementDAO.findRequiredByPrimaryKey(requirementId);
		Facility f = requirement.getFacilityScope();

		StringBuilder sb = new StringBuilder("INSERT INTO vss.VolunteerRequirement " + //
				" (VolunteerFK, RequirementFK, RequirementDate, Comments, Ver, CREATED_BY, CREATED_DATE, MODIFIED_BY, " //
				+ "MODIFIED_DATE, VSS_STD_RequirementStatusFK) "); //

		if (f == null) {
			// global, insert for all volunteers
			sb.append("select v.id, :requirementId, null ,null, 0,:myUser,:nowUTC,:myUser,:nowUTC, null" //:statusFK" 
					+ " from vss.Volunteers v"); //
			sb.append(" where not exists (select * from vss.VolunteerRequirement vr2") //
					.append(" where vr2.VolunteerFK = v.id") //
					.append(" and vr2.RequirementFK = :requirementId)");
		} else {

			// local, just for current facility
			sb.append(
					"(select distinct(va.VssVolunteersFK), :requirementId, null ,null, 0,:myUser,:nowUTC,:myUser,:nowUTC, null")//:statusFK") 
					.append(" from vss.VolunteerAssignments va") //
					.append(" join vss.Facility f on va.facilityFK = f.id") //
					.append(" where ((f.type = 'F' and f.id = :facilityId) or (f.type = 'L' and ParentFacilityFK = :facilityId))") //
					.append(" 	and not exists (select * from vss.VolunteerRequirement vr2") //
					.append(" 		where vr2.VolunteerFK = va.VssVolunteersFK") //
					.append(" 		and vr2.RequirementFK = :requirementId))");

			// ------------------ Insert those where primary facility is set

			sb.append("union (select v.id, :requirementId, null ,null, 0,:myUser,:nowUTC,:myUser,:nowUTC, null"// /*:statusFK*/" 
					+ " from vss.Volunteers v " //
					+ " join vss.Facility f on v.PrimaryFacilityFK = f.id" //
					+ " where ((f.type = 'F' and f.id = :facilityId) or (f.type = 'L' and ParentFacilityFK = :facilityId))"); //

			sb.append(" and not exists (select * from vss.VolunteerRequirement vr2") //
					.append(" where vr2.VolunteerFK = v.id") //
					.append(" and vr2.RequirementFK = :requirementId))");
		}

		Query q = em.createNativeQuery(sb.toString());

		q.setParameter("myUser", AbstractAuditedPersistent.getCurrentUserIdForAudit())
				.setParameter("nowUTC", ZonedDateTime.now(ZoneId.of("Z")));//.setParameter("statusFK", UNMET.getId());

		q.setParameter("requirementId", requirementId);
		if (f != null) {
			q.setParameter("facilityId", f.getId());
		}

		return q.executeUpdate();
	}

	@Override
	public int bulkAddNecessaryRequirements(long benefitingServiceRoleId) {
		flush();

		return em
				.createNativeQuery("insert into vss.VolunteerRequirement(VolunteerFK, RequirementFK, " //
						+ " Ver, CREATED_BY, CREATED_DATE, MODIFIED_BY, MODIFIED_DATE)" //
						+ " select va.VssVolunteersFK" //
						+ ", bsrr.RequirementFK" //
						+ ", 0" //
						+ ", :changeUser" //
						+ ", SYSUTCDATETIME()" //
						+ ", :changeUser" //
						+ ", SYSUTCDATETIME()" //
						+ " from vss.VolunteerAssignments va," //
						+ " vss.BenefitingServiceRoleRequirement bsrr" //
						+ "		join vss.Requirement r on bsrr.RequirementFK = r.id" //
						+ "	where bsrr.IsInactive = 0" //
						+ "		and r.IsInactive = 0" //
						+ "		and bsrr.BenefitingServiceRoleFK = :benefitingServiceRoleId" //
						+ "		and va.VssBenefitingServiceRolesFK = :benefitingServiceRoleId" //
						+ "		and not exists (select * from vss.VolunteerRequirement vr2" //
						+ "			where vr2.VolunteerFK = va.VssVolunteersFK" //
						+ "			and vr2.RequirementFK = bsrr.RequirementFK)") //
				.setParameter("benefitingServiceRoleId", benefitingServiceRoleId) //
				.setParameter("changeUser", AbstractAuditedPersistent.getCurrentUserIdForAudit()) //
				.executeUpdate();
	}

	@Override
	public int bulkRemoveUnnecessaryRequirements(long benefitingServiceRoleId) {
		// TODO CPB
		return 0;
	}
	
	@Override
	public int bulkUpdateDateToNull(long requirementId) {	
		String jpql = "update " + VolunteerRequirement.class.getName() //
		+ " set requirementDate = null  where requirement.id = :requirementId ";//
	
		Query q = query(jpql);
		
		q.setParameter("requirementId", requirementId);
		
		return q.executeUpdate();
	}

}
