package gov.va.vss.service.impl;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.google.common.collect.Lists;
import com.google.common.collect.Sets;
import com.google.common.collect.Sets.SetView;

import gov.va.vss.model.leie.ExcludedEntity;
import gov.va.vss.persistence.dao.leie.ExcludedEntityDAO;
import gov.va.vss.persistence.dao.leie.ExcludedEntityDAO.ImportDataCallback;
import gov.va.vss.service.ExcludedEntityService;

@Service
public class ExcludedEntityServiceImpl implements ExcludedEntityService {
	private static final Logger log = LoggerFactory.getLogger(ExcludedEntityServiceImpl.class);

	@Autowired
	private ExcludedEntityDAO excludedEntityDAO;
	@Autowired
	private ExcludedEntityServiceHelper helper;

	@Override
	public synchronized int refreshData() throws IOException {
		List<ExcludedEntity> all = excludedEntityDAO.findAll();
		HashSet<ExcludedEntity> existing = new HashSet<>(all);
		
		final HashSet<ExcludedEntity> newVals = new HashSet<>();
		int numResults = excludedEntityDAO.importData(new ImportDataCallback() {
			@Override
			public void processRecord(ExcludedEntity e) {
				newVals.add(e);
			}
		});
		
		SetView<ExcludedEntity> itemsToRemove = Sets.difference(existing, newVals);
		SetView<ExcludedEntity> itemsToAdd = Sets.difference(newVals, existing);
		boolean changed = false;

		for (List<ExcludedEntity> batch : Lists.partition(new ArrayList<>(itemsToAdd), 50)) {
			changed = true;
			helper.addValues(batch);
		}

		for (List<ExcludedEntity> batch : Lists.partition(new ArrayList<>(itemsToRemove), 50)) {
			changed = true;
			helper.deleteValues(batch);
		}

		helper.updateExecutedDate();
		if (changed)
			helper.updateDataChangedDate();

		return numResults;
	}

}
