package gov.va.vss.service.impl;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import gov.va.shared.service.validation.ServiceValidationException;
import gov.va.vss.model.requirement.AbstractRequirement;
import gov.va.vss.service.RequirementService;

@Service
public class RequirementServiceImpl extends AbstractServiceImpl implements RequirementService {
	private static final Logger log = LoggerFactory.getLogger(RequirementServiceImpl.class);

	@Override
	public AbstractRequirement saveOrUpdate(AbstractRequirement requirement, boolean isDateTypeChangedToNotApplicable) throws ServiceValidationException {
		boolean isNew = !requirement.isPersistent();
		
		if (isNew) {
			// prevent duplicate {name, facility}
			List<AbstractRequirement> potentialDuplicates = requirementDAO.findByCriteria(requirement.getFacilityScope() == null ? null : requirement.getFacilityScope().getId(), requirement.getName());
			if (!potentialDuplicates.isEmpty())
				throw new ServiceValidationException("requirement.error.duplicateNameFacility");
		} 
		
		AbstractRequirement r = requirementDAO.saveOrUpdate(requirement);
		if (isNew) {
			// for any new requirement added, trigger bulk insert of volunteer requirements
			volunteerRequirementDAO.bulkInsertForNewRequirement(r.getId());
		} else {
			// bulk update if date type is changed to Not Applicable
			if (isDateTypeChangedToNotApplicable) {
				//  When changing a date type on an existing requirement from any other value to ”Not Applicable”
				// all VolunteerRequirement dates associated to this requirement are deleted.
				volunteerRequirementDAO.bulkUpdateDateToNull(r.getId());
			}
		}
		
		return r;
	}
	
	@Override
	public void delete(long requirementId) {
		requirementDAO.delete(requirementId);
	}

}
