package gov.va.vss.util;

import javax.servlet.http.HttpSession;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import gov.va.shared.model.lookup.sds.VAFacility;
import gov.va.shared.persistence.lookup.sds.VAFacilityDAO;
import gov.va.shared.util.ServletUtil;
import gov.va.vss.model.facility.Facility;
import gov.va.vss.model.facility.Facility.FacilityValue;
import gov.va.vss.persistence.dao.facility.FacilityDAO;

@Component
public class SessionUtil {

	private static String KEY_PREFIX = SessionUtil.class.getName();
	public static final String CONTEXT_FACILITY_KEY = KEY_PREFIX + ".FACILITY";
	public static final String CONTEXT_FACILITY_NAME_KEY = KEY_PREFIX + ".FACILITY_NAME";
	public static final String CONTEXT_FACILITY_NUM_MEALS_KEY = KEY_PREFIX + ".FACILITY_NUM_MEALS";
	public static final String CONTEXT_FACILITY_IS_CO_KEY = KEY_PREFIX + ".FACILITY_IS_CENTRAL_OFFICE";

	@Autowired
	private VAFacilityDAO vaFacilityDAO;
	@Autowired
	private FacilityDAO facilityDAO;

	// ---------------------------------------- Static helper methods

	public static HttpSession getHttpSession() {
		return ServletUtil.getThreadBoundServletRequest().getSession(true);
	}

	public static Facility getFacilityContext() {
		return getFacilityContext(getHttpSession());
	}

	public static Facility getFacilityContext(HttpSession session) {
		return (Facility) session.getAttribute(CONTEXT_FACILITY_KEY);
	}

	public static String getFacilityContextName(HttpSession session) {
		return (String) session.getAttribute(CONTEXT_FACILITY_NAME_KEY);
	}

	public static Integer getFacilityContextNumMeals(HttpSession session) {
		return (Integer) session.getAttribute(CONTEXT_FACILITY_NUM_MEALS_KEY);
	}

	public static Boolean isFacilityContextCentralOffice(HttpSession session) {
		return (Boolean) session.getAttribute(CONTEXT_FACILITY_IS_CO_KEY);
	}

	public static void setFacilityContextName(Facility f, HttpSession session) {
		session.setAttribute(CONTEXT_FACILITY_NAME_KEY, f == null ? null : f.getDisplayName());
	}

	public static void setFacilityContextNumMeals(Facility f, HttpSession session) {
		session.setAttribute(CONTEXT_FACILITY_NUM_MEALS_KEY, f == null ? null
				: f.getStationParameters() == null ? null : f.getStationParameters().getNumberOfMeals());
	}

	public static void setFacilityContextIsCentralOffice(Facility f, HttpSession session) {
		session.setAttribute(CONTEXT_FACILITY_IS_CO_KEY,
				f == null ? null : FacilityValue.CENTRAL_OFFICE.getId() == f.getId());
	}

	public void setFacilityContext(VAFacility vaFacility, Facility f) {
		setFacilityContext(vaFacility, f, getHttpSession());
	}

	/*
	 * Prevents lazy loading exceptions for station params, or whatever else we
	 * might need - CPB
	 */
	@Transactional
	public void setFacilityContext(VAFacility vaFacility, Facility f, HttpSession session) {
		VAFacility attachedVaFacility = vaFacility == null ? null
				: vaFacilityDAO.findRequiredByPrimaryKey(vaFacility.getId());
		Facility attachedFacility = f == null ? null : facilityDAO.findRequiredByPrimaryKey(f.getId());

		gov.va.shared.util.SessionUtil.setSiteContext(attachedVaFacility, session);
		session.setAttribute(CONTEXT_FACILITY_KEY, attachedFacility);
		setFacilityContextName(attachedFacility, session);
		setFacilityContextNumMeals(attachedFacility, session);
		setFacilityContextIsCentralOffice(attachedFacility, session);
	}

}
