package gov.va.vss.service.scheduledJobs;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Profile;
import org.springframework.jmx.export.annotation.ManagedOperation;
import org.springframework.jmx.export.annotation.ManagedResource;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

@Service
@Profile(value = { "attended", "default" })
@ManagedResource
public class ExcludedEntityRefreshJob extends AbstractScheduledJob {
	private static final Logger log = LoggerFactory.getLogger(ExcludedEntityRefreshJob.class);

	@Value("${scheduledJobs.excludedEntityRefresh.enabled}")
	private boolean jobEnabled;

	@Scheduled(cron = "${scheduledJobs.excludedEntityRefresh.cron}")
	// or if it's on a delay, use this config:
	// @Scheduled(initialDelayString = "${scheduledJobs.startupDelayMillis}", //
	// fixedDelayString = "${scheduledJobs.cprsHeaderSync.fixedDelay}")
	@ManagedOperation
	public void refreshExcludedEntities() {
		if (!jobEnabled) {
			log.debug(ExcludedEntityRefreshJob.class.getSimpleName() + " not enabled; exiting");
			return;
		}

		log.info(Thread.currentThread().toString() + ": Running scheduled job {}",
				ExcludedEntityRefreshJob.class.getSimpleName());

		try {
			excludedEntityService.refreshData();
		} catch (Exception t) {
			log.error("Caught exception in " + ExcludedEntityRefreshJob.class.getName(), t);
		}
	}
}
