package gov.va.vss.service.scheduledJobs;

import java.time.ZoneId;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Profile;
import org.springframework.jmx.export.annotation.ManagedOperation;
import org.springframework.jmx.export.annotation.ManagedResource;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

@Service
@Profile(value = { "attended" })
@ManagedResource
public class VolunteerAutoInactivationJob extends AbstractScheduledJob {
	private static final Logger log = LoggerFactory.getLogger(VolunteerAutoInactivationJob.class);

	@Value("${scheduledJobs.volunteerAutoInactivation.switchToTimeZone}")
	private ZoneId switchToTimeZone;
	
	@Scheduled(cron = "${scheduledJobs.volunteerAutoInactivation.cron}")
	@ManagedOperation
	public void inactivateStaleVolunteers() {
		log.info(Thread.currentThread().toString() + ": Running scheduled job {}",
				VolunteerAutoInactivationJob.class.getSimpleName());
		int numInactivated = volunteerService.inactivateStaleVolunteers(switchToTimeZone);
		log.info("{} volunteers inactivated", numInactivated);
	}
}
