package gov.va.vss.web.requirement;

import java.util.ArrayList;
import java.util.List;
import java.util.SortedSet;

import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import com.fasterxml.jackson.annotation.JsonView;

import gov.va.shared.model.Permission;
import gov.va.shared.service.validation.ServiceValidationException;
import gov.va.vss.model.lookup.RequirementDateType;
import gov.va.vss.model.lookup.RequirementDateType.RequirementDateTypeValue;
import gov.va.vss.model.requirement.AbstractRequirement;
import gov.va.vss.model.requirement.AbstractRequirement.RequirementView;
import gov.va.vss.model.requirement.FacilityRequirement;
import gov.va.vss.model.requirement.GlobalRequirement;
import gov.va.vss.web.AbstractAppController;
import gov.va.vss.web.breadcrumbs.Breadcrumb;

@Controller
public class RequirementController extends AbstractAppController {

	@RequestMapping("/manageRequirements.htm")
	@Breadcrumb("Manage Requirements")
	@PreAuthorize("hasAnyAuthority('" + Permission.REQUIREMENTS_LOCAL_MANAGE +
	 ", " + Permission.REQUIREMENTS_GLOBAL_MANAGE + "')")
	public String manageRequirements(ModelMap model) {
		createReferenceData(model);
		model.addAttribute("DateValueNotApplicable", RequirementDateTypeValue.NOT_APPLICABLE);
		
		return "manageRequirements";
	}

/*	@RequestMapping("/getManageRequirementsStations")
	@JsonView(FacilityView.Basic.class)
	public @ResponseBody SortedSet<Facility> getStationsForManageRequirements() {
		Set<VAFacility> validFacilities = getCurrentUser()
				.getFacilitiesWhereUserHasAllPermissions(PermissionType.LOGIN_APPLICATION);
		Map<Long, Facility> facilitiesMap = facilityDAO.findByVAFacilities(validFacilities);
		return new TreeSet<>(facilitiesMap.values());
	}*/

	@RequestMapping("/requirements/local")
	@JsonView(RequirementView.Basic.class)
	public @ResponseBody List<FacilityRequirement> getFacilityRequirements() {
		List<AbstractRequirement> results = requirementDAO.findByCriteria(getFacilityContextId(), null);
		
		// ask Connor
		List<FacilityRequirement> facilityList = new ArrayList<FacilityRequirement>();
		for (AbstractRequirement r : results) {
			facilityList.add((FacilityRequirement) r);
		}
		
		return facilityList;
	}

	@RequestMapping("/requirements/global")
	@JsonView(RequirementView.Basic.class)
	public @ResponseBody SortedSet<GlobalRequirement> getGlobalRequirements() {
		SortedSet<GlobalRequirement> results = requirementDAO.findAllSortedByType(GlobalRequirement.class);
		return results;
	}

	@RequestMapping(value = "/requirement/saveOrUpdate", method = RequestMethod.POST)
	// @PreAuthorize("hasAuthority('" + Permission.VOLUNTEER_VIEW + "')")
	public @ResponseBody boolean requirementCreateOrUpdate(@RequestParam(required = false) Long requirementId, 
			@RequestParam(required = false) String facilityType,
			@RequestParam String name, @RequestParam String description,
			@RequestParam(required = false) RequirementDateType dateType,
			@RequestParam boolean isActive,
			@RequestParam boolean isDateTypeChangedToNotApplicable) throws ServiceValidationException {
		
		AbstractRequirement r;
		if (requirementId != null) {
			r = requirementDAO.findRequiredByPrimaryKey(requirementId);
		} else {
			if ("local".equals(facilityType)) {
				r = new FacilityRequirement();
				((FacilityRequirement)r).setFacility(getFacilityContext());
			} else {
				r = new GlobalRequirement();
			}
		}

		r.setName(name);
		r.setDescription(description);
		r.setDateType(dateType);
		r.setInactive(!isActive);

		r = requirementService.saveOrUpdate(r, isDateTypeChangedToNotApplicable);
		return true;
	}


	private void createReferenceData(ModelMap model) {
		model.put("allRequirementDateTypes", requirementDateTypeDAO.findAllSorted());
	}

}
