package gov.va.vss.web.volunteer;

import java.time.LocalDate;

import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;
import org.springframework.validation.Errors;

import gov.va.vss.model.Volunteer;
import gov.va.vss.util.DateUtil;
import gov.va.vss.util.SessionUtil;
import gov.va.vss.web.AbstractAppValidator;

@Component
public class VolunteerValidator extends AbstractAppValidator<VolunteerCommand> {

	@Override
	public void doExtraValidations(VolunteerCommand command, Errors errors) {
		Volunteer volunteer = command.getVolunteer();

		// for example: CPB

		// if (volunteer.getCurrentInactivationDate() != null
		// && DateTimeComparator.getInstance().compare(
		// volunteer.getCurrentActivationDate(),
		// volunteer.getCurrentInactivationDate()) > 0) {
		// errors.rejectValue("currentInactivationDate",
		// "currentInactivationDate.errors.invalidInactivationDate",
		// "The inactivation date must be later than the activation date");
		// }

		// or:

		// ValidationUtils.rejectIfEmpty(errors, field, errorCode, errorArgs,
		// defaultMessage);

		// (note that @NotBlank is Hibernate-specific API but achieves the same
		// as an annotation on the field)

		Integer mealsEligible = volunteer.getMealsEligible();
		if (mealsEligible != null && mealsEligible == 0 && StringUtils.isBlank(volunteer.getMealRemarks())) {
			errors.rejectValue("volunteer.mealRemarks", "volunteer.error.requiredMealRemarks");
		}

		if (command.isVolunteerTerminatedWithCause()) {
			if (command.getTerminationDate() == null)
				errors.rejectValue("terminationDate", "volunteer.error.requiredTerminationDate");
		}

		if (command.getTerminationDate() != null) {
			if (StringUtils.isBlank(command.getTerminationRemarks())) {
				errors.rejectValue("terminationRemarks", "volunteer.error.requiredTerminationRemarks");
			}
			if (command.getTerminationDate().isBefore(volunteer.getEntryDate())) {
				errors.rejectValue("terminationDate", "volunteer.error.terminationDateBeforeEntryDate");
			} else {
				LocalDate mostRecentWorkEntryDate = command.getTimeSummary().getMostRecentWorkEntryDate();
				if (mostRecentWorkEntryDate != null && !command.getTerminationDate().isAfter(mostRecentWorkEntryDate)) {
					errors.rejectValue("terminationDate",
							"volunteer.error.terminationDateBeforeMostRecentWorkEntryDate",
							new Object[] { mostRecentWorkEntryDate.format(DateUtil.DATE_ONLY_FORMAT) }, "");
				}
			}
		}

		LocalDate dateOfBirth = volunteer.getDateOfBirth();
		if (dateOfBirth != null && dateOfBirth.isAfter(LocalDate.now(SessionUtil.getFacilityContext().getTimeZone()))) {
			errors.rejectValue("volunteer.dateOfBirth", "volunteer.error.dateOfBirthNotInFuture");
		}

		LocalDate lastAwardDate = volunteer.getLastAwardDate();
		if (lastAwardDate != null && lastAwardDate.isBefore(volunteer.getEntryDate())) {
			errors.rejectValue("volunteer.lastAwardDate", "volunteer.error.lastAwardDateBeforeEntryDate");
		}

	}

}
