package gov.va.vss.persistence.impl;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import javax.persistence.Query;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Repository;

import gov.va.vss.model.donation.DonGenPostFund;
import gov.va.vss.persistence.dao.DonGenPostFundDAO;
import gov.va.vss.persistence.queryCustomization.QueryCustomization;

@Repository
public class DonGenPostFundDAOImpl extends GenericHibernateSortedDAOImpl<DonGenPostFund> implements DonGenPostFundDAO {
	@SuppressWarnings("unused")
	private static final Logger log = LoggerFactory.getLogger(DonGenPostFundDAOImpl.class);

	/*
	 * @Override public List<DonGenPostFund> findByCriteria(Long id,
	 * QueryCustomization... customization) { return null; }
	 */

	@SuppressWarnings("unchecked")
	@Override
	public DonGenPostFund findByProgramCode(String gpfCode, long facilityId) {

		List<String> whereClauseItems = new ArrayList<>();
		Map<String, Object> params = new HashMap<>();

		StringBuilder sb = new StringBuilder("select gpf from ").append(DonGenPostFund.class.getName()).append(" gpf");
		sb.append(" left join fetch gpf.facility f");

		if (gpfCode != null) {
			whereClauseItems.add("TRIM(LOWER(gpf.generalPostFund)) like :gpfCode");
			params.put("gpfCode", gpfCode.toLowerCase().trim() + "%");
		}
		if (facilityId > 0) {
			whereClauseItems.add("f.id = :facilityId");
			params.put("facilityId", facilityId);
		}

		Query q = constructQuery(em, sb, whereClauseItems, params, null);
		List<DonGenPostFund> results = q.getResultList();

		return results.isEmpty() ? null : results.get(0);
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<DonGenPostFund> findByFacility(long facilityId) {
		StringBuilder sb = new StringBuilder("select d from ").append(DonGenPostFund.class.getName()).append(" d");
		sb.append(" left join fetch d.facility f");

		QueryCustomization cust = new QueryCustomization();

		if (cust.getOrderBy() == null)
			cust.setOrderBy("d.generalPostFund");

		List<String> whereClauseItems = new ArrayList<>();
		Map<String, Object> params = new HashMap<>();

		if (facilityId > 0) {
			whereClauseItems.add("d.facility.id = :facilityId");
			params.put("facilityId", facilityId);
		}

		Query q = constructQuery(em, sb, whereClauseItems, params, null, cust);

		return q.getResultList();
	}

	@SuppressWarnings("unchecked")
	@Override
	public Map<Long, BigDecimal[]> getDonGenPostFundReportByFacility(long facilityId, LocalDate startDateInclusive,
			LocalDate endDateInclusive) {
		Query q = em.createNativeQuery("with donation_sums as (" //
				+ "		select gpf.id, total = sum(dd.DonationValue)" //
				+ "		from vss.DonGenPostFund gpf" //
				+ "			left join vss.DonationSummary ds on gpf.FacilityFK = ds.FacilityFK" //
				+ "				and ds.DonationDate >= :dateStartInclusive" //
				+ "				and ds.DonationDate <= :dateEndInclusive" //
				+ "			left join vss.DonationDetail dd on ds.id = dd.DonationSummaryFK" //
				+ "				and dd.GeneralPostFundFK = gpf.id" //
				+ "		where gpf.FacilityFK = :facilityId" //
				+ "		group by gpf.id" //
				+ "	)," //
				+ "	expenditure_sums as (" //
				+ "		select gpf.id, total = sum(e.Amount)" //
				+ "		from vss.DonGenPostFund gpf" //
				+ "			left join vss.Expenditure e on gpf.FacilityFK = e.FacilityFK" //
				+ "				and e.GeneralPostFundFK = gpf.id" //
				+ "				and e.RequestDate >= :dateStartInclusive" //
				+ "				and e.RequestDate <= :dateEndInclusive" //
				+ "		where gpf.FacilityFK = :facilityId" //
				+ "		group by gpf.id" //
				+ "	)," //
				+ "	ledger_adjustment_sums as (" //
				+ "		select gpf.id, total = sum(e.Amount)" //
				+ "		from vss.DonGenPostFund gpf" //
				+ "			left join vss.LedgerAdjustment e on gpf.FacilityFK = e.FacilityFK" //
				+ "				and e.GeneralPostFundFK = gpf.id" //
				+ "				and e.RequestDate >= :dateStartInclusive" //
				+ "				and e.RequestDate <= :dateEndInclusive" //
				+ "		where gpf.FacilityFK = :facilityId" //
				+ "		group by gpf.id" //
				+ "	)" //
				+ "	select gpf.id, gpf.GeneralPostFund," //
				+ "		donation_total = ISNULL(ds.total, 0)," //
				+ "		expenditure_total = ISNULL(es.total, 0)," //
				+ "		ledger_adjustment_total = ISNULL(las.total, 0)" //
				+ "	from vss.DonGenPostFund gpf" //
				+ "		left join donation_sums ds on gpf.id = ds.id" //
				+ "		left join expenditure_sums es on gpf.id = es.id" //
				+ "		left join ledger_adjustment_sums las on gpf.id = las.id" //
				+ "	where gpf.FacilityFK = :facilityId" //
				+ "	order by gpf.GeneralPostFund");
		q.setParameter("dateStartInclusive", startDateInclusive);
		q.setParameter("dateEndInclusive", endDateInclusive);
		q.setParameter("facilityId", facilityId);

		Map<Long, BigDecimal[]> results = new LinkedHashMap<>();

		List<Object[]> rows = q.getResultList();
		for (Object[] row : rows) {
			long gpfId = ((Number) row[0]).longValue();
			BigDecimal donationTotal = (BigDecimal) row[2];
			BigDecimal expenditureTotal = (BigDecimal) row[3];
			BigDecimal ledgerAdjustmentTotal = (BigDecimal) row[4];
			results.put(gpfId, new BigDecimal[] { donationTotal, expenditureTotal, ledgerAdjustmentTotal });
		}
		return results;
	}

}
