package gov.va.vss.web.ledger;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.temporal.TemporalAdjusters;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import gov.va.vss.model.donation.DonGenPostFund;
import gov.va.vss.web.AbstractAppController;
import gov.va.vss.web.breadcrumbs.Breadcrumb;
import gov.va.vss.web.expenditure.ExpenditureController;

@Controller
public class LedgerController extends AbstractAppController {

	@Autowired
	private ExpenditureController expenditureController;
	
	@RequestMapping("/ledgerSummary")
	public @ResponseBody List<GPFSummary> getSummary(@RequestParam String dateRangeType) {
		LocalDate startDate = null;
		LocalDate endDate = null;
		if ("fy".equals(dateRangeType)) {
			startDate = dateUtil.getCurrentFiscalYearStartDate(getFacilityTimeZone());
			endDate = dateUtil.getCurrentFiscalYearEndDate(getFacilityTimeZone());
		} else if ("lastfy".equals(dateRangeType)) {
			startDate = dateUtil.getPreviousFiscalYearStartDate(getFacilityTimeZone());
			endDate = dateUtil.getPreviousFiscalYearEndDate(getFacilityTimeZone());
		} else if ("month".equals(dateRangeType)) {
			startDate = LocalDate.now().withDayOfMonth(1);
			endDate = LocalDate.now();
		} else if ("lastmonth".equals(dateRangeType)) {
			startDate = LocalDate.now().withDayOfMonth(1).minusMonths(1);
			endDate = startDate.with(TemporalAdjusters.lastDayOfMonth());
		} else {
			throw new IllegalArgumentException("Unsupported date range type '" + dateRangeType + "'");
		}

		List<GPFSummary> results = getGPFBalancesInternal(startDate, endDate);
		return results;
	}

	@RequestMapping("/donGenPostFund/balances")
	public @ResponseBody List<GPFSummary> getGPFBalances() {
		return getGPFBalancesInternal(LocalDate.of(1900, 1, 1), LocalDate.now());
	}

	private List<GPFSummary> getGPFBalancesInternal(LocalDate startDate, LocalDate endDate) {
		List<GPFSummary> results = new ArrayList<>();
		List<DonGenPostFund> gpfs = donGenPostFundDAO.findByFacility(getFacilityContextId());
		Map<Long, BigDecimal[]> gpfSums = donGenPostFundDAO.getDonGenPostFundReportByFacility(getFacilityContextId(),
				startDate, endDate);
		for (DonGenPostFund gpf : gpfs) {
			BigDecimal[] sums = gpfSums.get(gpf.getId());
			BigDecimal donationSum = sums == null ? BigDecimal.ZERO : sums[0];
			BigDecimal expenditureSum = sums == null ? BigDecimal.ZERO : sums[1];
			BigDecimal ledgerAdjustmentSum = sums == null ? BigDecimal.ZERO : sums[2];
			results.add(new GPFSummary(gpf, donationSum, expenditureSum, ledgerAdjustmentSum));
		}
		return results;
	}

	@RequestMapping("/ledger.htm")
	@Breadcrumb("General Ledger")
	// @PreAuthorize("hasAuthority('" + Permission.EDONATION_MANAGE + "')")
	public String showGeneralLedger(ModelMap model) {
		createReferenceData(model);
		// setFormAsReadOnlyUnlessUserHasPermissions(model,
		// PermissionType.STAFF_TITLE_CREATE);
		return "ledger";

	}

	private void createReferenceData(ModelMap model) {
		expenditureController.populateReferenceDataForPopup(model);
	}
}
