<%@ include file="../shared/inc_header.jsp"%>

<jsp:include page="../shared/inc_expenditureDetail.jsp">
	<jsp:param name="resultCallbackMethod" value="refreshTable" />
	<jsp:param name="getExpenditureMapFn" value="getExpenditureMap" />
</jsp:include>

<script type="text/javascript">
	$(function() {
		$('.dateInput').each(function() {
			$(this).enableDatePicker({
				showOn : "button",
				buttonImage : imgHomePath + "/calendar.gif",
				buttonImageOnly : true
			})
			$(this).mask(twoDigitDateMask)
		})

		var theDataTable = $('#expenditureSearchResultsList').DataTable({
			buttons : [ {
				extend : 'excel'
			}, {
				extend : 'pdfHtml5',
				orientation : 'landscape'
			}, {
				extend : 'print'
			} ],

			"columns" : [ {
				"render" : function(row, type, val, meta) {
					return val.donGenPostFund.generalPostFund
				}
			}, {
				"render" : function(row, type, val, meta) {
					if (type === 'display')
						return '<a href="javascript:editExpenditure(' + val.id + ')">' + val.requestDate + '</a>'
					return val.requestDate
				}
			}, {
				"render" : function(row, type, val, meta) {
					if (type == 'display')
						return '$' + formatAndAddThousandSeparators(val.amount)
					return val.amount
				}
			}, {
				"render" : function(row, type, val, meta) {
					return val.description
				}
			}, {
				"render" : function(row, type, val, meta) {
					return val.vendor
				}
			}, {
				"render" : function(row, type, val, meta) {
					var s = val.quantity
					if (val.unit.code != 'EA')
						s += ' ' + val.unit.name
					s += ' @ $' + formatAndAddThousandSeparators(val.unitPrice) + (val.quantity > 1 ? '/' + val.unit.name : '')
					return s
				}
			}, {
				"render" : function(row, type, val, meta) {
					return val.originator.displayName
				}
			}, {
				"render" : function(row, type, val, meta) {
					return val.transactionId
				}
			}, {
				"render" : function(row, type, val, meta) {
					return val.purchaseOrderNumber
				}
			}, {
				"render" : function(row, type, val, meta) {
					return val.comments
				}
			} ],
			"dom" : '<"top"fBi>rt<"bottom"><"clear">',
			"order" : [],
			"paging" : false
		})
		
		$("#generalPostFundSelect").val(${donGenPostFundId})
		$("#beginDateInput").val('${beginDate}')
		$("#endDateInput").val('${endDate}')

		$("#submitButton").click(refreshTable)
		refreshTable()
	})

	function validateForm() {
		$("input").prop("disabled", false)
		showSpinner(null, true)
		return true
	}

	var expenditureMap = {}
	
	function refreshTable() {
		$.ajax({
			url : ajaxHomePath + '/expendature/list',
			dataType : 'json',
			data : {
				"beginDate" : $("#beginDateInput").val(),
				"endDate" : $("#endDateInput").val(),
				"donGenPostFundId" : $("#generalPostFundSelect").val()
			},
			error : commonAjaxErrorHandler,
			success : function(response) {
				expenditureMap = {}
				for (var i = 0; i < response.length; i++)
					expenditureMap[response[i].id] = response[i]
				
				var table = $('#expenditureSearchResultsList').DataTable()
				table.clear()
				table.rows.add(response)
				table.draw()
				
				$("#listWrapper").toggle(response.length > 0)
				$("#noResultsDiv").toggle(response.length == 0)
				$("#maxResultsDiv").toggle(response.length == ${maxResults})
			}
		})
	}
	
	function getExpenditureMap() {
		return expenditureMap
	}
</script>

<div class="clearCenter">
	<table>
		<tr>
			<td align="right">Begin Date:</td>
			<td><input size="12" id="beginDateInput" class="dateInput" /></td>
			<td width="40" rowspan="2">&nbsp;</td>
			<td align="right">General Post Fund:</td>
			<td><select id="generalPostFundSelect">
					<option value="-1">(any)</option>
					<c:forEach items="${allGPFs}" var="gpf">
						<option value="${gpf.id}"><c:out
								value="${gpf.generalPostFund}" /></option>
					</c:forEach>
			</select></td>
		</tr>
		<tr>
			<td align="right">End Date:</td>
			<td><input size="12" id="endDateInput" class="dateInput" /></td>
			<td></td>
			<td></td>
		</tr>
	</table>

	<div class="clearCenter" style="padding-top: 15px">
		<a id="submitButton" class="buttonAnchor" href="#">Submit</a> <a
			id="cancelOperationBtn" class="buttonAnchor"
			href="${current_breadcrumb}">Cancel</a>
	</div>
</div>

<style>
.ui-state-default a.tableHeaderLink, .ui-state-default a.tableHeaderLink:link,
	.ui-state-default a.tableHeaderLink:visited {
	text-decoration: underline;
	font-weight: normal;
}

#expenditureSearchResultsList thead th.tableHeaderLinkWrapper {
	font-weight: normal;
}
#expenditureSearchResultsList {
	border-collapse: collapse;
}
#expenditureSearchResultsList td {
    padding: 3px;
}
</style>

<div class="clearCenter" style="padding-top: 15px; display: none"
	id="maxResultsDiv">The maximum number of search results was met.
	Please add more restrictive criteria and search again.</div>
<div class="clearCenter" style="padding-top: 15px; display: none"
	id="noResultsDiv">
	Sorry, no expenditures were found that matched the specified criteria.
	</td>
</div>
<div class="clearCenter" id="listWrapper" style="display:none">
	<table id="expenditureSearchResultsList" class="stripe" border="1"
		summary="List of Expenditures">
		<thead>
			<tr>
				<th>General Post Fund</th>
				<th>Date</th>
				<th>Amount</th>
				<th>Description</th>
				<th>Vendor</th>
				<th>Quantity</th>
				<th>Originator</th>
				<th>Transaction #</th>
				<th>Purchase Order</th>
				<th>Comments</th>
			</tr>
		</thead>
		<tbody>
		</tbody>
	</table>
</div>